@@ -316,14 +316,14 @@ int nvidia_p2p_init_mapping(
     return -ENOTSUPP;
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_init_mapping);
+EXPORT_SYMBOL(nvidia_p2p_init_mapping);
 
 int nvidia_p2p_destroy_mapping(uint64_t p2p_token)
 {
     return -ENOTSUPP;
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_destroy_mapping);
+EXPORT_SYMBOL(nvidia_p2p_destroy_mapping);
 
 static void nv_p2p_mem_info_free_callback(void *data)
 {
@@ -506,13 +506,8 @@ static int nv_p2p_get_pages(
     (*page_table)->page_size = page_size_index;
 
     os_free_mem(physical_addresses);
-    physical_addresses = NULL;
-
     os_free_mem(wreqmb_h);
-    wreqmb_h = NULL;
-
     os_free_mem(rreqmb_h);
-    rreqmb_h = NULL;
 
     if (free_callback != NULL)
     {
@@ -587,7 +582,7 @@ int nvidia_p2p_get_pages(
                             p2p_token, va_space, virtual_address,
                             length, page_table, free_callback, data);
 }
-NV_EXPORT_SYMBOL(nvidia_p2p_get_pages);
+EXPORT_SYMBOL(nvidia_p2p_get_pages);
 
 int nvidia_p2p_get_pages_persistent(
     uint64_t virtual_address,
@@ -605,7 +600,7 @@ int nvidia_p2p_get_pages_persistent(
                             virtual_address, length, page_table,
                             NULL, NULL);
 }
-NV_EXPORT_SYMBOL(nvidia_p2p_get_pages_persistent);
+EXPORT_SYMBOL(nvidia_p2p_get_pages_persistent);
 
 /*
  * This function is a no-op, but is left in place (for now), in order to allow
@@ -618,7 +613,7 @@ int nvidia_p2p_free_page_table(struct nvidia_p2p_page_table *page_table)
     return 0;
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_free_page_table);
+EXPORT_SYMBOL(nvidia_p2p_free_page_table);
 
 int nvidia_p2p_put_pages(
     uint64_t p2p_token,
@@ -645,12 +640,11 @@ int nvidia_p2p_put_pages(
     status = nv_p2p_put_pages(NV_P2P_PAGE_TABLE_TYPE_NON_PERSISTENT,
                               sp, p2p_token, va_space,
                               virtual_address, &page_table);
-
     nv_kmem_cache_free_stack(sp);
 
     return nvidia_p2p_map_status(status);
 }
-NV_EXPORT_SYMBOL(nvidia_p2p_put_pages);
+EXPORT_SYMBOL(nvidia_p2p_put_pages);
 
 int nvidia_p2p_put_pages_persistent(
     uint64_t virtual_address,
@@ -690,7 +684,7 @@ int nvidia_p2p_put_pages_persistent(
 
     return nvidia_p2p_map_status(status);
 }
-NV_EXPORT_SYMBOL(nvidia_p2p_put_pages_persistent);
+EXPORT_SYMBOL(nvidia_p2p_put_pages_persistent);
 
 int nvidia_p2p_dma_map_pages(
     struct pci_dev *peer,
@@ -805,7 +799,7 @@ failed:
     return nvidia_p2p_map_status(status);
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_dma_map_pages);
+EXPORT_SYMBOL(nvidia_p2p_dma_map_pages);
 
 int nvidia_p2p_dma_unmap_pages(
     struct pci_dev *peer,
@@ -845,7 +839,7 @@ int nvidia_p2p_dma_unmap_pages(
     return 0;
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_dma_unmap_pages);
+EXPORT_SYMBOL(nvidia_p2p_dma_unmap_pages);
 
 /*
  * This function is a no-op, but is left in place (for now), in order to allow
@@ -860,7 +854,7 @@ int nvidia_p2p_free_dma_mapping(
     return 0;
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_free_dma_mapping);
+EXPORT_SYMBOL(nvidia_p2p_free_dma_mapping);
 
 int nvidia_p2p_register_rsync_driver(
     nvidia_p2p_rsync_driver_t *driver,
@@ -889,7 +883,7 @@ int nvidia_p2p_register_rsync_driver(
                                     driver->wait_for_rsync, data);
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_register_rsync_driver);
+EXPORT_SYMBOL(nvidia_p2p_register_rsync_driver);
 
 void nvidia_p2p_unregister_rsync_driver(
     nvidia_p2p_rsync_driver_t *driver,
@@ -921,7 +915,7 @@ void nvidia_p2p_unregister_rsync_driver(
                                driver->wait_for_rsync, data);
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_unregister_rsync_driver);
+EXPORT_SYMBOL(nvidia_p2p_unregister_rsync_driver);
 
 int nvidia_p2p_get_rsync_registers(
     nvidia_p2p_rsync_reg_info_t **reg_info
@@ -1014,7 +1008,7 @@ int nvidia_p2p_get_rsync_registers(
     return 0;
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_get_rsync_registers);
+EXPORT_SYMBOL(nvidia_p2p_get_rsync_registers);
 
 void nvidia_p2p_put_rsync_registers(
     nvidia_p2p_rsync_reg_info_t *reg_info
@@ -1046,4 +1040,4 @@ void nvidia_p2p_put_rsync_registers(
     os_free_mem(reg_info);
 }
 
-NV_EXPORT_SYMBOL(nvidia_p2p_put_rsync_registers);
+EXPORT_SYMBOL(nvidia_p2p_put_rsync_registers);
